#ifndef BARRIERE_H
#define BARRIERE_H

#include <Arduino.h>

/******************************************************************
classe abstraite representant une barriere
la barriere peut etre commandee de differentes facons  (bobines, relais, servo, moteur pas a pas)
constructeur :
     tepms : une duree de manoeuvre
     sorte : le type de barrieres unipole (1), dipole(2), quadripole(4)
attributs utilisables par tous les heritiers
    mode sens (enum mode {montee,descente,arret}), sens de la manoeuvre
    state situation (enum state {ouvert, ferme, encours}), etat de la barriere
    int pinOpen : pin pour l'ouverture
    int pinClose : pin pour la fermeture
    int duree : une duree d'activation
Methodes utilisables par tous
    bool ouverte() : vrai si barriere ouverte
    bool fermee() : vrai si barriere fermee
    bool stop() : vrai si pas de manoeuvre
    bool monte() : vrai si en cours de montee
    bool descend() : vrai si en cours de descente
    bool mouvement() : vrai si manoeuvre active
 Methodes a redefinir pour chaque type de barriere
        void ouverture() : methode qui commande l'ouverture de la barriere
        void fermeture() : methode qui commande la fermeture de la barriere
        void manoeuvre() : methode qui commande le deplacement de la barriere
*****************************************************************/

class barriere
{
    public:
        // constructeurs
        barriere();
        barriere(int temps,byte sorte);
        // destructeur
        virtual ~barriere();
        // etat ouvert de la barriere
        bool ouverte() {return (situation == ouvert)?true:false;}
        // etat ferme de la barriere
        bool fermee() {return (situation == ferme)?true:false;}
        // etat arret du mouvement
        bool arretee() {return (sens == arret)?true:false;}
        // etat mouvement montant
        bool monte() {return (sens == montee)?true:false;}
        // etat mouvement descendant
        bool descend() {return (sens == descente)?true:false;}
        // etat mouvement en cours
        bool mouvement() {return _actif;}
        // retourne si la barriere est active
        bool actif() {return _actif;}
        // retourne le type de vbarrieres
        byte nombrePoles() {return poles;}
        // methode a redefinir qui retourne les pins utilisees
        virtual int broche (int num) = 0;
        // methode abstraite d'ouverture de la barriere
        virtual void ouverture() = 0;
        // methode abstraite de fermeture de la barriere
        virtual void fermeture() = 0;
        // methode abstraite de manoeuvre de la barriere
        virtual void manoeuvre() = 0;


    protected:
        // les differents sens de la barriere
        enum mode {montee,descente,arret};
        // attribut du sens de la barriere
        mode sens = arret;
        // les differents etats de la barrierre
        enum etat {ouvert, ferme, encours};
        // l'etat courant de la barriere
        etat situation = ouvert;
        // etat actif de la barriere
        bool _actif = false;
        // timer de manoeuvre
        unsigned long timer;
        // temps d'activation de la barriere
        int duree;
        // type de barrieres
        byte poles;        // vrai 2 pins, faux 4 pins
    private:
};

#endif // BARRIERE_H
