#ifndef VOIE_H
#define VOIE_H

#include "zone.h"
#include <Arduino.h>

#define GAUCHE 0
#define DROITE 1

/********************************************************
la classe commune a toutes les voies
elle represente un ensemble de 4 zones de capteurs
zone gauche ouvrante, zone gauche fermante, zone droite ouvrante, zone droite fermante
constructeur :
        rien
        un nom (optionnel)
Methodes :
    char* getName() : renvoie le nom de la voie
    int getState(): renvoie l'etat de la voie, nombre de demandes de passage
    bool demandeFermeture() : renvoie si il y a une demande de fermeture (vrai dde, faux sinon)
    void config(zone1,zone2,zone3,zone4,nom(optionnel)) : les instances des 4 zones, un nom (optionnel)
    bool print() : affiche la configuration
    bool Loop : boucle sur les capteurs et algo de la voie
****************************************************************************/

class voie
{
    private:
    // nom de la voie
    char* _nom;
    // demande de fermeture
    bool _demandeFermeture = false;
    
    // positionne l'etat de la voie
    //void setState(int state) {_state = state;}
    
    // etat des demandes de la passage pour la voie
    int _etat = 0;

    
    // positionne la direction courante     (direction du train, vers la droite (RIGHT) ou vers la gauche (LEFT))
    //void directionTrain(bool dir){_dir = dir;}
    // renvoie la direction courante du train (LEFT ou RIGHT)
    //bool getDir() {return _dir;}

    // structure pour les zones ouvrantes et fermantes
    typedef struct detecteur {
      zone *ouvre;
      zone *ferme;
    };
    // un element de la liste des zones
    typedef struct element {
    detecteur zoneGauche;
    detecteur zoneDroite;
    };
    // un voie = 2 cantons (gauche, droit)
    element canton[2];
    // direction courante de la voie
    bool directionTrain = GAUCHE;
    // mise a jour de tous les capteurs de la voie
    bool actualiser();
    // algorithme d'ouverture
    void algoOuvre(bool);
    // algorithme de fermeture
    void algoFerme(bool);

    public:
        // les constructeurs
        voie();
        voie(char*);
        // le destructeur
        virtual ~voie();
        // configuration de la voie
        void config (zone &, zone &, zone &, zone &);// zoneGaucheOuvrante, zoneGaucheFermante, zoneDroiteOuvrante, zoneDroiteFermante
        void config(zone &, zone &, zone &, zone &, char*);  // idem avec un nom pour la voie
        void config(zone &, zone &, zone &); // voie unidirectionnelle zoneGauche, zoneCentre, zoneDroite
        void config(zone &, zone &, zone &,char*); // idem avec un nom
        // renvoie le nom
        char* nom(){return _nom;}
        // renvoie l'etat de la voie
        int etat(){return _etat;}
        // renvoie si une demande de fermeture a lieu
        bool demandeFermeture() {return _demandeFermeture;}
        // affichage de la configuration
        void print();
        // boucle sur la configuration de la voie
        void Loop();
    protected:

};
#endif // VOIE_H
